/*
    8 and 9 October 2014:

    Animation experiment using CSS3. See
    http://www.w3schools.com/css/css3_animations.asp
    http://www.w3schools.com/cssref/css3_pr_animation.asp

    (Rapid Eye Movement;
     EMDR: Eye Movement Desensitization and Reprocessing?)

    10 October 2014:
    Not only fully random colours, but also some named ones
    (also randomly chosen) as an alternative.

    10 October 2014:
    Use a better randomiser function where available (Unix)
*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>

#define NumSteps       32
#define MovesPerSecond 0.555556

#if defined(_Windows) || defined (__MSDOS__)
  #define DRAND() ((double)rand()   / (double)RAND_MAX)
  #define SRAND() (srand  ((unsigned)time(NULL)))
#else
  #include <sys/types.h>
  #include <unistd.h>
  #define DRAND() ((double)random() / (double)0x7fffffffL)
  #define SRAND() (srandom((unsigned)time(NULL) + getpid()))
#endif

static char code1[] =
"<!DOCTYPE html>\n"
"<html>\n"
"<head>\n"
"<style>\n"
"body { background: black; }\n"
"\n"
"div {                     \n"
"    width:     12px;      \n"
"    height:    12px;      \n"
"    background: red;      \n"
"    position: absolute;   \n"
"    top:0; left:0;        \n"
"    animation: RapidEye %ds;             \n"
"    animation-iteration-count: infinite; \n"
"    animation-timing-function: ease-out; \n"
"}\n"
"\n"
;

static char code2[] =
"/* Standard syntax */\n"
"@keyframes RapidEye {\n"
;

static char code2lines[] =
"    %4.1f%% { top:%2d%%; left:%2d%%; background: #%02x%02x%02x;}\n"
;

static char code2linesNamed[] =
"    %4.1f%% { top:%2d%%; left:%2d%%; background: %-7s;}\n"
;

static char code3[] =
"}\n"
;

static char code4[] =
"</style>\n"
"</head>\n"
"<body>\n"
"<div></div>\n"
"</body>\n"
"</html>\n"
;

static char *ColorTab[] =
{
   "aqua", "black", "blue", "fuchsia", "gray", "green",
   "lime", "maroon", "navy", "olive", "orange", "purple",
   "red", "silver", "teal", "white", "yellow"
};

static int RandomPatterns(void);
static int PrintLines (void);

int main (void)
{
   int d;

   printf("Content-type: text/html\n\n");
   fflush(stdout);

   d = (NumSteps / (double)MovesPerSecond) + 0.5;
   printf(code1, d, d);
   RandomPatterns();
   printf(code4);

   return 0;
}

static int RandomPatterns(void)
{
   SRAND();

   printf(code2);
   PrintLines();
   printf(code3);

   return 0;
}

int PrintLines (void)
{
   int i;
   float perc;

   for (i = 0, perc = 0.0; i < NumSteps && perc <= 100.0; i++)
   {
      if ((float) DRAND() < 0.34)
      {
         printf(code2lines, perc,
            (int)(0.5 + DRAND() * 95),
            (int)(0.5 + DRAND() * 95),
            (int)(0.5 + DRAND() * 0xff),
            (int)(0.5 + DRAND() * 0xff),
            (int)(0.5 + DRAND() * 0xff));
      }
      else
      {
         int index = (int)(DRAND() * sizeof ColorTab / (double)sizeof ColorTab[0]);

         printf(code2linesNamed, perc,
            (int)(0.5 + DRAND() * 95),
            (int)(0.5 + DRAND() * 95),
            ColorTab[index]);
      }
      perc += 100.0 / NumSteps;
   }

   return 0;
}
